package com.loafle.overflow.central.module.sensor.service;

import com.loafle.overflow.central.module.meta.service.CentralMetaCrawlerMappingService;
import com.loafle.overflow.central.module.meta.service.CentralMetaDisplayItemMappingService;
import com.loafle.overflow.central.module.sensor.dao.SensorDAO;
import com.loafle.overflow.central.module.target.service.CentralTargetService;
import com.loafle.overflow.core.exception.OverflowException;
import com.loafle.overflow.model.meta.MetaCrawlerMapping;
import com.loafle.overflow.model.meta.MetaDisplayItemMapping;
import com.loafle.overflow.model.meta.MetaSensorStatus;
import com.loafle.overflow.model.sensor.Sensor;
import com.loafle.overflow.model.target.Target;
import com.loafle.overflow.service.central.sensor.SensorService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * Created by insanity on 17. 6. 28.
 */
@Service("SensorService")
public class CentralSensorService implements SensorService {

  @Autowired
  private SensorDAO sensorDAO;
  @Autowired
  private CentralSensorItemService sensorItemService;
  @Autowired
  private CentralTargetService targetService;
  @Autowired
  private CentralMetaCrawlerMappingService metaCrawlerMappingService;
  @Autowired
  private CentralMetaDisplayItemMappingService metaDisplayItemMappingService;
  @Autowired
  private CentralSensorConfigService sensorConfigService;

  public void registDefaults(List<Target> targets) throws OverflowException {
    for (Target target : targets) {
      this.registDefault(target);
    }
  }

  private void registDefault(Target target) throws OverflowException {
    List<MetaCrawlerMapping> metaCrawlerMappings = this.metaCrawlerMappingService
        .readAllByMetaTargetTypeKey(target.getMetaTargetType().getKey());
    for (MetaCrawlerMapping metaCrawlerMapping : metaCrawlerMappings) {
      if (metaCrawlerMapping.getDefault()) {
        Sensor sensor = new Sensor();
        sensor.setMetaCrawlerMapping(metaCrawlerMapping);
        sensor.setInterval(metaCrawlerMapping.getDefaultInterval());
        sensor.setTarget(target);

        this.regist(sensor, this.metaDisplayItemMappingService.readAllDefault(metaCrawlerMapping.getId()));
      }
    }
  }

  @Transactional
  @Override
  public Sensor regist(Sensor sensor, List<MetaDisplayItemMapping> metaDisplayItemMappings) throws OverflowException {
    Long targetID = sensor.getTarget().getId();
    Target t = this.targetService.read(targetID);
    if (null == t) {
      throw new OverflowException(String.format("ID[%d] of Target is not valid", targetID));
    }
    if (null == sensor.getName()) {
      String crawlerName = sensor.getMetaCrawlerMapping().getMetaCrawler().getName();
      String targetTypeName = sensor.getTarget().getMetaTargetType().getName();
      sensor.setName(targetTypeName + " Sensor via " + crawlerName);
    }
    sensor.setMetaSensorStatus(MetaSensorStatus.Enum.RUNNING.to());
    Sensor retSensor = this.sensorDAO.save(sensor);
    retSensor.setItemCount(0);
    this.targetService.increaseSensorCount(targetID);

    for (MetaDisplayItemMapping metaDisplayItemMapping : metaDisplayItemMappings) {
      this.sensorItemService.regist(retSensor, metaDisplayItemMapping);
    }

    this.sensorConfigService.regist(retSensor);

    return retSensor;
  }

  @Transactional
  @Override
  public Sensor modify(Long sensorID, List<MetaDisplayItemMapping> metaDisplayItemMappings) throws OverflowException {
    Sensor sensor = this.read(sensorID);
    if (null == sensor) {
      throw new OverflowException(String.format("ID[%d] of Sensor is not valid", sensorID));
    }
    this.sensorItemService.removeAllBySensorID(sensorID);
    for (MetaDisplayItemMapping metaDisplayItemMapping : metaDisplayItemMappings) {
      this.sensorItemService.regist(sensor, metaDisplayItemMapping);
    }
    return sensor;
  }

  @Override
  public Sensor read(Long id) throws OverflowException {
    return this.sensorDAO.findById(id).get();
  }

  @Transactional
  @Override
  public void remove(Long id) throws OverflowException {
    Sensor sensor = this.read(id);
    if (null == sensor) {
      throw new OverflowException(String.format("ID[%d] of Sensor is not valid", id));
    }
    this.targetService.decreaseSensorCount(sensor.getTarget().getId());
    this.sensorDAO.deleteById(id);
  }

  @Override
  public List<Sensor> readAll() throws OverflowException {
    return this.sensorDAO.findAll();
  }

  @Override
  public List<Sensor> readAllByTargetID(Long targetID) throws OverflowException {
    return this.sensorDAO.findAllByTargetId(targetID);
  }

  @Override
  public List<Sensor> readAllByTargetIn(List<Target> targets) throws OverflowException {
    return this.sensorDAO.findAllByTargetIn(targets);
  }

  @Override
  public List<Sensor> readAllByTargetInfraProbeID(Long probeID) throws OverflowException {
    return this.sensorDAO.findAllByTargetInfraProbeId(probeID);
  }

  public Sensor regist(Sensor sensor) throws OverflowException {
    Long targetID = sensor.getTarget().getId();
    Target t = this.targetService.read(targetID);
    if (null == t) {
      throw new OverflowException(String.format("ID[%d] of Target is not valid", targetID));
    }
    return this.sensorDAO.save(sensor);
  }

  public Sensor increaseSensorItemCount(Long id) throws OverflowException {
    Sensor s = this.sensorDAO.findById(id).get();
    s.setItemCount(s.getItemCount() + 1);
    return this.sensorDAO.save(s);
  }

  public Sensor decreaseSensorItemCount(Long id) throws OverflowException {
    Sensor s = this.sensorDAO.findById(id).get();
    int count = s.getItemCount();
    if (count > 0) {
      s.setItemCount(count - 1);
    }
    return this.sensorDAO.save(s);
  }

}